import { COLS_PER_PAGE } from "../base/constants.js";
import { GroupingProviderFactory } from "./rowGroupingProvider.js";
import { createAgGridPage } from "./rowFactory.js";
import { areTotalsChanged, isInvalidGetRowsRequest } from "./dataSourceUtils.js";
import isEqual from "lodash/isEqual.js";
import { attributeLocalId, bucketAttribute, bucketSetTotals, bucketsFind, defTotals, dimensionSetTotals, isAttributeSort, } from "@gooddata/sdk-model";
import { BucketNames, DataViewFacade, emptyHeaderTitleFromIntl } from "@gooddata/sdk-ui";
import { TableDescriptor } from "../structure/tableDescriptor.js";
export function createAgGridDatasource(config, initialDv, gridApiProvider, intl, getCurrentAbortController) {
    return new AgGridDatasource(config, initialDv, gridApiProvider, intl, getCurrentAbortController);
}
export class AgGridDatasource {
    config;
    initialDv;
    gridApiProvider;
    intl;
    getCurrentAbortController;
    rowCount;
    destroyed = false;
    currentResult;
    currentSorts;
    grouping;
    constructor(config, initialDv, gridApiProvider, intl, getCurrentAbortController) {
        this.config = config;
        this.initialDv = initialDv;
        this.gridApiProvider = gridApiProvider;
        this.intl = intl;
        this.getCurrentAbortController = getCurrentAbortController;
        this.currentResult = initialDv.result();
        this.currentSorts = initialDv.meta().effectiveSortItems();
        // we do not have a sortModel yet so we need to check the dataView itself if the sorting makes sense
        const isInitialDvSortedByFirstAttribute = isDataViewSortedByFirstAttribute(initialDv);
        this.grouping = this.createGroupingProvider(isInitialDvSortedByFirstAttribute);
    }
    createGroupingProvider = (isSortedByFirst) => {
        // grouping happens under two conditions: it is desired & the data is sorted by first column
        const shouldGroup = this.config.getGroupRows() && isSortedByFirst;
        return GroupingProviderFactory.createProvider(shouldGroup);
    };
    resetGroupingProvider = (sortModel = []) => {
        const isSortedByFirst = isSortedByFirstAttribute(this.config.tableDescriptor, sortModel);
        this.grouping = this.createGroupingProvider(isSortedByFirst);
    };
    onDestroy = () => {
        //
    };
    processData = (dv, params) => {
        if (!dv) {
            return;
        }
        const { successCallback } = params;
        const pageData = createAgGridPage(dv, this.config.tableDescriptor, this.intl);
        const { rowData, rowTotals } = pageData;
        const { offset, count, totalCount } = dv.dataView;
        const rowAttributeIds = this.config.tableDescriptor.headers.sliceCols.map((header) => header.id);
        this.grouping.processPage(rowData, offset[0], rowAttributeIds);
        const columnHeadersPosition = this.config.getColumnHeadersPosition();
        // RAIL-1130: Backend returns incorrectly total: [1, N], when count: [0, N] and offset: [0, N]
        const rowsCount = columnHeadersPosition === "left" ? rowData.length : totalCount[0];
        // Tiger backend returns incorrectly count when no measures are present count: [1, N]
        // Check if there are no measures or row attributes present to display
        const shouldNotHaveRows = this.config.tableDescriptor.getMeasures().length === 0 && rowAttributeIds.length === 0;
        const lastRow = (offset[0] === 0 && count[0] === 0) || shouldNotHaveRows ? 0 : rowsCount;
        this.config.onPageLoaded(dv);
        successCallback(rowData, lastRow);
        /**
         * In version 25 ag-grid has much better logic that detects which cells need to be refreshed.
         * Unfortunately, this breaks row grouping because cells outside of the currently loaded page are not redrawn.
         * This is a problem, because the newly loaded page might change the grouping status of cells outside of the loaded page.
         * So we force a cell refresh to redraw all the cells with the up-to-date grouping CSS classes.
         * This basically reverts the improved logic in the new ag-grid and behaves very closely to ag-grid 22.
         */
        this.gridApiProvider()?.refreshCells({ force: true, suppressFlash: true });
        // set totals
        if (areTotalsChanged(this.gridApiProvider(), rowTotals)) {
            this.gridApiProvider()?.updateGridOptions({ pinnedBottomRowData: rowTotals });
        }
    };
    transformResult = (params, desiredSorts, desiredTotals, desiredRowTotals) => {
        const { sortModel } = params;
        const result = this.currentResult;
        const definition = result.definition;
        /*
         * This seems to trigger re-render of column/row headers, thus ensuring that sort indicators
         * are shown correctly.
         */
        this.gridApiProvider()?.refreshHeader();
        /*
         * Grouping provider is stateful, keeps accumulating info about pages; now that the pages
         * fundamentally change due to re-exec, the grouping provider must be reset as well.
         */
        this.resetGroupingProvider(sortModel);
        const abortController = this.getCurrentAbortController();
        const transformedResult = abortController ? result.withSignal(abortController.signal) : result;
        let transformedExecution = transformedResult
            .transform()
            .withSorting(...(desiredSorts ?? []))
            .withDimensions(dimensionSetTotals(definition.dimensions[0], desiredTotals), dimensionSetTotals(definition.dimensions[1], desiredRowTotals));
        const columnBucket = bucketsFind(definition.buckets, BucketNames.COLUMNS);
        const attributeBucket = bucketsFind(definition.buckets, BucketNames.ATTRIBUTE);
        // Update buckets property only when adding or removing columns totals/subtotals. So table is updated properly.
        if (desiredRowTotals.length > 0 || (desiredRowTotals && columnBucket?.totals !== undefined)) {
            if (attributeBucket) {
                transformedExecution = transformedExecution.withBuckets(definition.buckets[0], bucketSetTotals(definition.buckets[1], desiredTotals), bucketSetTotals(definition.buckets[2], desiredRowTotals));
            }
            else {
                transformedExecution = transformedExecution.withBuckets(definition.buckets[0], bucketSetTotals(definition.buckets[1], desiredRowTotals));
            }
        }
        this.config.onExecutionTransformed(transformedExecution);
        this.driveExecutionAndUpdateDatasource(transformedExecution, params);
    };
    driveExecutionAndUpdateDatasource = (execution, params) => {
        const { startRow, endRow, failCallback } = params;
        let effectiveExecution = execution;
        const abortController = this.getCurrentAbortController();
        if (abortController) {
            effectiveExecution = execution.withSignal(abortController.signal);
        }
        effectiveExecution
            .execute()
            .then((newResult) => {
            this.currentResult = newResult;
            newResult
                .readWindow([startRow, 0], [endRow - startRow, COLS_PER_PAGE])
                .then((data) => {
                const dataView = this.config.dataViewTransform(data);
                const dv = DataViewFacade.for(dataView);
                this.gridApiProvider()?.setRowCount(dataView.totalCount[0]);
                this.currentSorts = dv.meta().effectiveSortItems();
                // Table descriptors contain information about effective totals (e.g. totals set for the
                // table right now). After redrive of execution to change sorts/totals, code must make
                // sure that the new settings are reflected in the table descriptor.
                const emptyValue = emptyHeaderTitleFromIntl(this.intl);
                this.config.tableDescriptor = TableDescriptor.for(dv, emptyValue, {
                    columnHeadersPosition: this.config.tableDescriptor.hasHeadersOnLeft()
                        ? "left"
                        : "top",
                }, this.intl);
                this.processData(dv, params);
            })
                .catch((err) => {
                this.config.onTransformedExecutionFailed();
                console.error("Error while doing execution to obtain data view", err);
                failCallback();
            });
        })
            .catch((err) => {
            this.config.onTransformedExecutionFailed();
            console.error("Error while doing execution to obtain transformed results", err);
            failCallback();
        });
    };
    destroy = () => {
        if (this.destroyed) {
            return;
        }
        this.destroyed = true;
        this.onDestroy();
    };
    getRows = (params) => {
        const { startRow, endRow, failCallback, sortModel } = params;
        if (isInvalidGetRowsRequest(startRow, this.gridApiProvider())) {
            failCallback();
            return;
        }
        const result = this.currentResult;
        const definition = result.definition;
        const desiredSorts = this.config.tableDescriptor.createSortItems(sortModel ?? [], result.definition.sortBy);
        const currentTotals = defTotals(definition, 0);
        const desiredTotals = this.config.getColumnTotals();
        const currentRowTotals = defTotals(definition, 1);
        const desiredRowTotals = this.config.getRowTotals();
        if (!isEqual(this.currentSorts, desiredSorts) ||
            !isEqual(currentTotals, desiredTotals) ||
            !isEqual(currentRowTotals, desiredRowTotals)) {
            // Due to timing issues in react18, transform result needs to be deferred so that remaining
            // re-renders can finish in the meantime -- this fixes the cases in AD: immediate undo after
            // performing sort and also repeated sort on the same attribute (also note that due to
            // other timeouts, setTimeout(...,0) is not enough).
            setTimeout(() => {
                this.transformResult(params, desiredSorts, desiredTotals, desiredRowTotals);
            }, 100);
        }
        else if (!startRow && result.definition === this.initialDv.definition) {
            /*
             * > Loading first page of data
             *
             * AND
             *
             * > Exec definition of the effective result to get data from is same as the exec definition of the
             * initial data view that was used to construct the table and its data source.
             *
             * ===
             *
             * can reuse the initial data view
             *
             */
            this.processData(this.initialDv, params);
        }
        else {
            result
                .readWindow([startRow, 0], [endRow - startRow, COLS_PER_PAGE])
                .then((data) => {
                const dataView = this.config.dataViewTransform(data);
                this.processData(DataViewFacade.for(dataView), params);
            })
                .catch((err) => {
                console.error("Error while doing execution to obtain data view", err);
                failCallback();
            });
        }
    };
    getGroupingProvider = () => {
        return this.grouping;
    };
}
function isSortedByFirstAttribute(tableDescriptor, sortingCols) {
    if (!sortingCols.length) {
        // this is somewhat dangerous assumption: no explicit sort == sorted by first col
        //  (the backend behaves thusly)
        return true;
    }
    if (sortingCols.length > 1) {
        return false;
    }
    return tableDescriptor.isFirstCol(sortingCols[0].colId);
}
function isDataViewSortedByFirstAttribute(dv) {
    const { sortBy } = dv.definition;
    if (!sortBy?.length) {
        // this is somewhat dangerous assumption: no explicit sort == sorted by first col
        //  (the backend behaves thusly)
        return true;
    }
    if (sortBy.length > 1) {
        return false;
    }
    const firstSortBy = sortBy[0];
    if (!isAttributeSort(firstSortBy)) {
        return false;
    }
    const attributeBucket = dv.def().bucket(BucketNames.ATTRIBUTE);
    if (!attributeBucket) {
        return false;
    }
    const firstAttribute = bucketAttribute(attributeBucket);
    if (!firstAttribute) {
        return false;
    }
    return firstSortBy.attributeSortItem.attributeIdentifier === attributeLocalId(firstAttribute);
}
//# sourceMappingURL=dataSource.js.map