// (C) 2021-2022 GoodData Corporation
import { isScopeCol } from "./tableDescriptorTypes.js";
import { isResultTotalHeader, } from "@gooddata/sdk-model";
import { invariant } from "ts-invariant";
import zip from "lodash/zip.js";
function createAttributeLocator(descriptor, header) {
    // if this bombs it means something is wrong with the col descriptors or in very bad case in the DVF data access logic.
    // by contract, all data series have same number of descriptors & headers. therefore the zipping logic should never
    // run into situation where the arrays are of different size.
    invariant(descriptor && header);
    return {
        attributeLocatorItem: {
            attributeIdentifier: descriptor.attributeHeader.localIdentifier,
            element: header.attributeHeaderItem.uri,
        },
    };
}
function createTotalLocator(descriptor, header) {
    // if this bombs it means something is wrong with the col descriptors or in very bad case in the DVF data access logic.
    // by contract, all data series have same number of descriptors & headers. therefore the zipping logic should never
    // run into situation where the arrays are of different size.
    invariant(descriptor && header);
    return {
        totalLocatorItem: {
            attributeIdentifier: descriptor.attributeHeader.localIdentifier,
            totalFunction: header.totalHeaderItem.name,
        },
    };
}
function createMeasureLocator(descriptor) {
    return {
        measureLocatorItem: {
            measureIdentifier: descriptor.measureHeaderItem.localIdentifier,
        },
    };
}
/**
 * Given a leaf data col, this function will create column locator that can be used in
 * width items.
 *
 * @param col - col definition to get locators for
 */
export function createColumnLocator(col) {
    if (isScopeCol(col)) {
        const result = [];
        const { descriptorsToHere, headersToHere, measureDescriptors } = col;
        const descriptorsAndHeaders = zip(descriptorsToHere, headersToHere);
        descriptorsAndHeaders.push([col.attributeDescriptor, col.header]);
        descriptorsAndHeaders.forEach(([descriptor, header]) => isResultTotalHeader(header)
            ? result.push(createTotalLocator(descriptor, header))
            : result.push(createAttributeLocator(descriptor, header)));
        if (measureDescriptors) {
            measureDescriptors.forEach((measureDescriptor) => result.push(createMeasureLocator(measureDescriptor)));
        }
        return result;
    }
    else {
        const result = [];
        if (col.seriesDescriptor.attributeDescriptors && col.seriesDescriptor.attributeHeaders) {
            const descriptorAndHeaders = zip(col.seriesDescriptor.attributeDescriptors, col.seriesDescriptor.attributeHeaders);
            descriptorAndHeaders.forEach(([descriptor, header]) => {
                if (isResultTotalHeader(header)) {
                    result.push(createTotalLocator(descriptor, header));
                }
                else {
                    result.push(createAttributeLocator(descriptor, header));
                }
            });
        }
        result.push(createMeasureLocator(col.seriesDescriptor.measureDescriptor));
        return result;
    }
}
/**
 * Given a transposed slice measure or mixed values data col, this function will create column locator that can be used in
 * width items.
 *
 * @param col - col definition to get locators for
 */
export function createTransposedColumnLocator(col) {
    const result = [];
    if (col.seriesDescriptor) {
        const filteredSeriesDescriptor = col.seriesDescriptor.filter((serieDescriptor, index, self) => self.findIndex((s) => s.measureDescriptor === serieDescriptor.measureDescriptor) === index);
        filteredSeriesDescriptor.forEach((serieDescriptor) => {
            result.push(createMeasureLocator(serieDescriptor.measureDescriptor));
        });
    }
    return result;
}
//# sourceMappingURL=colLocatorFactory.js.map