import React, { useEffect, useState, useCallback } from "react";
import cx from "classnames";
import HeaderCell, { ALIGN_LEFT, ALIGN_RIGHT } from "./HeaderCell.js";
import { isEmptyScopeCol, isSliceCol, isSliceMeasureCol, isMixedHeadersCol, isMixedValuesCol, } from "../tableDescriptorTypes.js";
import { isEnterKey, isSpaceKey } from "@gooddata/sdk-ui-kit";
const ColumnHeader = (props) => {
    const { className, column, api, getTableDescriptor, displayName, enableSorting, menu, progressSort, eGridHeader, setLastSortedColId, } = props;
    const [sorting, setSorting] = useState(column.getSort());
    // Only sync sort state locally; sorted column id is tracked at table level.
    useEffect(() => {
        const syncSortState = (_event) => {
            const currentSort = column.getSort();
            setSorting(currentSort);
        };
        column.addEventListener("sortChanged", syncSortState);
        return () => {
            column.removeEventListener("sortChanged", syncSortState);
        };
    }, [column]);
    const onSortRequested = useCallback(() => {
        const multiSort = false;
        progressSort(multiSort);
        api.refreshHeader();
    }, [api, progressSort]);
    const getColDescriptor = () => {
        return props.getTableDescriptor().getCol(props.column);
    };
    const col = getColDescriptor();
    const textAlign = isSliceCol(col) ||
        isEmptyScopeCol(col) ||
        isSliceMeasureCol(col) ||
        isMixedValuesCol(col) ||
        isMixedHeadersCol(col)
        ? ALIGN_LEFT
        : ALIGN_RIGHT;
    const isColumnAttribute = isEmptyScopeCol(col);
    const isSortingEnabled = !isColumnAttribute &&
        !isSliceMeasureCol(col) &&
        !isMixedValuesCol(col) &&
        !isMixedHeadersCol(col) &&
        enableSorting;
    const tableDescriptor = getTableDescriptor();
    const showMenu = tableDescriptor.isTransposed()
        ? (isSliceMeasureCol(col) && displayName) || isMixedHeadersCol(col)
        : true;
    const [isFocused, setIsFocused] = useState(false);
    useEffect(() => {
        if (!api) {
            return;
        }
        const handleFocusChange = (event) => {
            const focusedColId = event.column.isColumn ? event.column.getColId() : event.column.getGroupId();
            const myColId = column.getColId();
            setIsFocused(focusedColId === myColId);
        };
        api.addEventListener("headerFocused", handleFocusChange);
        return () => {
            api.removeEventListener("headerFocused", handleFocusChange);
        };
    }, [api, column]);
    useEffect(() => {
        if (!eGridHeader) {
            return;
        }
        const handleKeyDown = (event) => {
            // set last sorted column id to table only when sort was triggered by keyboard
            if (isEnterKey(event) &&
                isSortingEnabled &&
                setLastSortedColId) {
                setLastSortedColId(column.getColId());
            }
            if (isSpaceKey(event)) {
                event.preventDefault();
                if (isSortingEnabled && setLastSortedColId) {
                    setLastSortedColId(column.getColId());
                    onSortRequested();
                }
            }
        };
        const handleBlur = () => {
            setIsFocused(false);
        };
        eGridHeader.addEventListener("keydown", handleKeyDown);
        if (!showMenu) {
            eGridHeader.addEventListener("blur", handleBlur);
        }
        return () => {
            eGridHeader.removeEventListener("keydown", handleKeyDown);
            if (!showMenu) {
                eGridHeader.removeEventListener("blur", handleBlur);
            }
        };
    }, [eGridHeader, column, isSortingEnabled, setLastSortedColId, showMenu, onSortRequested]);
    return (React.createElement(HeaderCell, { className: cx("s-pivot-table-column-header", className), textAlign: textAlign, displayText: displayName, enableSorting: isSortingEnabled, sortDirection: sorting, defaultSortDirection: column.getColDef().sortingOrder?.[0] ?? undefined, onSortClick: onSortRequested, onMenuAggregationClick: props.onMenuAggregationClick, menu: showMenu ? menu?.() : undefined, colId: column.getColDef().field, getTableDescriptor: getTableDescriptor, getExecutionDefinition: props.getExecutionDefinition, getColumnTotals: props.getColumnTotals, getRowTotals: props.getRowTotals, intl: props.intl, isFocused: isFocused }));
};
export default ColumnHeader;
//# sourceMappingURL=ColumnHeader.js.map